import java.lang.reflect.Field;
import java.awt.*;
import javax.swing.*;

public class EditBase extends JPanel {

    static final int LEFT = 0;
    static final int RIGHT = 1;
    static final int CENTER = 2;

    static final int T_STRING = 0;
    static final int T_INTEGER = 1;
    static final int T_ARRAY = 2;

    private static final int MAX = 50;
	
    private GridBagLayout layout;
    private Font font;

    private String[] names;
    private Object[] lists;
    private int[] types;
    private int[] index;

    private int y;			// current position
    private int n;			// number of items

    /*** Constructer ************************************************/

    public EditBase(){
	y = 0;
	n = 0;
	names = new String[ MAX ];
	lists = new Component[ MAX ];
	types = new int[ MAX ];
	index = new int[ MAX ];
	layout = new GridBagLayout();
	font = new Font("Serif", Font.PLAIN, 12 );

	setLayout(layout);
    }

    /*** Output Data ************************************************/

    public void getData( Object obj ){
	try{
	    for( int i=0; i < n; i++ ){
		Field field = obj.getClass().getDeclaredField(names[i]);
		switch( types[i] ){
		case T_STRING:
		    JTextField tf = (JTextField)lists[i];
		    field.set( obj, tf.getText() );
		    break;
		case T_INTEGER:
		    JComboBox cb = (JComboBox)lists[i];
		    field.setInt( obj, cb.getSelectedIndex() );
		    break;
		case T_ARRAY:
		    JComboBox cb2 = (JComboBox)lists[i];
		    int[] array = (int[])field.get(obj);
		    array[ index[i] ] = cb2.getSelectedIndex();
		    break;
		}
	    }
	}catch( Exception e ){
	    System.out.println(e);
	}
    }

    /*** Input Data *****************************************************/		

    public void setData( Object obj ){
	try{
	    for( int i=0; i < n; i++ ){
		Field field = obj.getClass().getDeclaredField(names[i]);
		switch( types[i] ){
		case T_STRING:
		    JTextField tf = (JTextField)lists[i];
		    tf.setText( (String)field.get(obj) );
		    break;
		case T_INTEGER:
		    JComboBox cb = (JComboBox)lists[i];
		    cb.setSelectedIndex( field.getInt(obj) );
		    break;
		case T_ARRAY:
		    JComboBox cb2 = (JComboBox)lists[i];
		    int[] array = (int[])field.get(obj);
		    cb2.setSelectedIndex( array[ index[i] ] );
		    break;
		}
	    }
	}catch( Exception e ){
	    System.out.println(e);
	}
    }

    /*** Deploy ********************************************************/

    private void add( int type, int i, Component comp,
		      String name, String label, int pos )
    {
	types[n] = type;
	index[n] = i;
	names[n] = name;
	lists[n] = comp;
	n++;

	JLabel labe = new JLabel(label);
	labe.setFont(font);
	labe.setForeground( Color.black );
	switch( pos ){
	case CENTER:
	    y++;
	    add( labe,	1, y, 1, 1 );
	    add( comp,  2, y, 3, 1 );
	    break;
	case LEFT:
	    y++;
	    add( labe,  1, y, 1, 1 );
	    add( comp,  2, y, 1, 1 );
	    break;
	case RIGHT:
	    add( labe,  3, y, 1, 1 );
	    add( comp,  4, y, 1, 1 );
	    break;
	}
    }

    /*** ComboBox ****************************************************/

    private JComboBox createJComboBox(){
	JComboBox cb = new JComboBox();
	cb.setFont(font);
	return cb;
    }

    /*** Set Field **********************************************/

    public void set( int pos, String name, String label ){
	JTextField tf = new JTextField(3);
	tf.setFont(font);
	tf.setBackground( Color.white );
	tf.setForeground( Color.black );
	add( T_STRING, 0, tf, name, label, pos );
    }

    /*** Set Choice ************************************************/

    public void set( int pos, String name, String label, String[] s ){
	JComboBox cb = createJComboBox();
	setItem( cb, s );
	add( T_INTEGER, 0, cb, name, label, pos );
    }
    public void set( int pos, String name, String label, int max ){
	JComboBox cb = createJComboBox();
	setItem( cb, max );
	add( T_INTEGER, 0, cb, name, label, pos );
    }

    /*** Deploy Setup  ********************************************/

    public void set( int pos, String name, int i, String label, String[] s ){
	JComboBox cb = createJComboBox();
	setItem( cb, s );
	add( T_ARRAY, i, cb, name, label, pos );
    }

    public void set( int pos, String name, int i, String label, int max ){
	JComboBox cb = createJComboBox();
	setItem( cb, max );
	add( T_ARRAY, i, cb, name, label, pos );
    }

    /*** Layout **************************************************/

    private void add( Component obj, int x, int y, int xs, int ys ){
	GridBagConstraints c = new GridBagConstraints();
	c.gridx = x;
	c.gridy = y;
	c.gridwidth = xs;
	c.gridheight = ys;
	c.weightx = 1;
	c.weighty = 0;
	c.fill = GridBagConstraints.HORIZONTAL;
	layout.setConstraints( obj, c );
	add( obj );
    }

    /*** Set Item ************************************************/

    private static void setItem( JComboBox cb, int max ){
	for( int i=0; i<=max; i++ ){
	    cb.addItem( "" + i );
	}
    }
    private static void setItem( JComboBox cb, String[] s ){
	if( s == null ){
	    setItem( cb, 50 );
	    return;
	}
	for( int i=0; i<s.length; i++ ){
	    cb.addItem( s[i] );
	}
    }
}
