package mine;

import javax.swing.*;
import java.awt.*;
import java.io.*;
import java.net.*;
import java.awt.image.*;
import java.util.*;
import java.awt.geom.AffineTransform;

public class Mine {

    public static final int METAL = 1;
    public static final int MOTIF = 2;
    public static final int WINDOWS = 3;

    /*** Font *******************************************************/

    public static Font getFont( int size ){
	return new Font("Dialog", Font.PLAIN, size );
    }

    /*** Windows ****************************************************/

    public static boolean isWindows(){
	String osname = System.getProperty("os.name");
	if ( osname.length() < 7 ) return false;
	return osname.substring(0,7).equals("Windows"); 
    }

    /*** Look&Feel **************************************************/

    public static void setLookAndFeel( int type ){
	try{
	    switch(type){
	    case METAL:
		String metal = "javax.swing.plaf.metal.MetalLookAndFeel";
		UIManager.setLookAndFeel(metal);
		break;
	    case MOTIF:
		String motif = "com.sun.java.swing.plaf.motif.MotifLookAndFeel";
		UIManager.setLookAndFeel(motif);
		break;
	    case WINDOWS:
		String windows = "com.sun.java.swing.plaf.windows.WindowsLookAndFeel";
		UIManager.setLookAndFeel(windows);
		break;
	    }
	}catch( Exception e ){
	    System.out.println(e);
	}
    }

    /*** Read String from File ***************************/

    public static String[] readStrings( String file ){
	try{
	    InputStream is = file.getClass().getResourceAsStream(file);
	    if( is == null ) is = new FileInputStream(file);

	    InputStreamReader isr = new InputStreamReader(is);
	    BufferedReader r = new BufferedReader(isr);
	    Vector list = new Vector();
	    while(true){
		String s = r.readLine();
		if( s == null ) break;
		list.add(s);
	    }
	    return (String[])list.toArray( new String[1] );
	}catch( Exception e ){
	    System.out.println(e);
	    return null;
	}
    }
   		
    /*** Output Error to File *************************************/

    public static void setErr( String filename){
	try{
	    FileOutputStream fos = new FileOutputStream(filename);
	    System.setErr( new PrintStream(fos) );
	}catch( Exception e ){
	    System.out.println(e);
	}
    }

    /*** Draw String on Center *******************************************/

    public static void drawString( String s, int x, int y, int xs, Graphics g ){
	Font f = g.getFont();
	int width = g.getFontMetrics(f).stringWidth(s);
	g.drawString( s, x + (xs - width)/2, y );
    }

    /*** Middle of Three Numbers *****************************/

    public static int mid( int s, int m, int b ){
	return Math.max( s, Math.min( m, b ) );
    }

    /*** Locate on Center *********************************************/

    public static void setCenter( Window w ){
	Dimension s = Toolkit.getDefaultToolkit().getScreenSize();
	Dimension f = w.getSize();
	f.height = Math.min( f.height, s.height );
	f.width = Math.min( f.width, s.width );
	w.setLocation((s.width - f.width) / 2, (s.height - f.height) / 2);
    }

    /*** Sprite *******************************************************/

    public static void setSprite( BufferedImage img, int r, int g, int b ){
	WritableRaster wr = img.getRaster();
	for( int x=0; x<wr.getWidth(); x++ ){
	    for( int y=0; y<wr.getHeight(); y++ ){
		int[] p = new int[4]; 
		wr.getPixel( x, y, p ); 
		if( p[0] == r && p[1] == g && p[2] == b ){
		    p[3] = 0;
		    wr.setPixel( x, y, p );
		}
	    }
	}
    }

    /*** Set Alpha *****************************************************/

    public static void setAlpha( Graphics g, double alpha ){
	AlphaComposite ac = 
	    AlphaComposite.getInstance( AlphaComposite.SRC_OVER, (float)alpha );
	((Graphics2D)g).setComposite(ac);
    }

    /*** Antialias ****************************************************/

    public static void setAntialias( Graphics g, boolean flag ){
	Object alias;
	if(flag){
	    alias = RenderingHints.VALUE_TEXT_ANTIALIAS_ON;
	}else{
	    alias = RenderingHints.VALUE_TEXT_ANTIALIAS_OFF;
	}
	RenderingHints.Key key = RenderingHints.KEY_TEXT_ANTIALIASING;
	RenderingHints hints = new RenderingHints( key, alias );
	((Graphics2D)g).setRenderingHints(hints);
    }

    /*** Draw a part of Image ********************************************/

    public static void drawImage
	( Image img, int ax, int ay, int bx, int by, int w, int h, Graphics g )
    {
	g.drawImage( img, ax, ay, ax+w, ay+h, bx, by, bx+w, by+h, null );
    }

    /*** Rotate and Draw ********************************************/

    public static void drawRotateImage
	( Image img, int ax, int ay, int bx, int by, int w, int h, 
	  double theta, Graphics g )
    {
	Graphics2D g2 = (Graphics2D)g;
	AffineTransform at = g2.getTransform();
	g2.rotate( theta, ax+w/2, ay+h/2 );
	g2.drawImage( img, ax, ay, ax+w, ay+h, bx, by, bx+w, by+h, null );
	g2.setTransform(at);
    }

    /*** Rescale and Draw ************************************************/

    public static BufferedImage getScaleImage( Image img, double Rate, boolean alpha ){
	int width = (int)(img.getWidth(null) * Rate );
	int height = (int)(img.getHeight(null) * Rate );
	return ImageLoader.getBuffer( img, width, height, alpha );
    }

    /*** Clip Image ********************************************/

    public static BufferedImage getClipImage
	( Image source, int x, int y, int xs, int ys, boolean alpha )
    {
	BufferedImage bimg = ImageLoader.createImage( xs, ys, alpha );
	Graphics g = bimg.getGraphics();
	g.drawImage( source, 0, 0, xs, ys, x, y, x+xs, y+ys, null );
	return bimg;
    }

    /*** Rotate Image *****************************************/

    public static BufferedImage getRotateImage
	( Image source, int x, int y, int w, int h, double theta )
    {
	BufferedImage bimg = ImageLoader.createImage( w, h, true );
	Graphics2D g = (Graphics2D)bimg.getGraphics();
	g.rotate( theta, w/2, h/2 );
	g.drawImage( source, 0, 0, w, h, x, y, x+w, y+h, null );
	return bimg;
    }

    /*** Reflect Angle ****************************/

    public static double refAngle( double anga, double angb ){
	double angle = anga - angb;
	while( angle < 0 ) angle += Math.PI*2;
	while( angle >= Math.PI*2 ) angle -= Math.PI*2;
	if( angle <= Math.PI/2 ) return anga;
	if( angle >= Math.PI*3/2 ) return anga;

	angle = angb*2 - anga - Math.PI;
	while( angle < 0 ) angle += Math.PI*2;
	while( angle >= Math.PI*2 ) angle -= Math.PI*2;
	return angle;
    }
}
